import { type APIRoute } from "astro";
import validateAccessToken from "../../../lib/utils/validateAccessToken";
import { createOrUpgradeResearch, getUserByAccessToken, getUserResearch } from "../../../lib/db/users";
import { getPlanetById } from "../../../lib/db/planets";
import Planet from "../../../types/Planet";
import { ObjectId } from "mongodb";
import { calculateCurrentAvailableResources, getResourcesFromPlanet, updatePlanetResources } from "../../../lib/utils/resourceManager";
import PlayerResource from "../../../types/PlayerResource";
import calculateAvailableResources from "../../../lib/utils/calculateAvailableResources";
import { getAllResearch, getResearchById } from "../../../lib/db/research";

export const POST: APIRoute = async({ request }) => {
    const response = await validateAccessToken(request);
    if(response instanceof Response) return response;

    const user = await getUserByAccessToken(response);
    if(user === null) {
        return new Response(
            JSON.stringify({
                code: 401,
                message: "Unauthorized"
            }), { status: 401 }
        )
    }

    let body;
    try {
        body = await request.json()
    } catch(e) {
        return new Response(
            JSON.stringify({
                code: 400,
                message: "Bad Request",
                error: "Invalid JSON body"
            }), { status: 400 }
        )
    }

    
    const researchId = body.research;
    const research = await getResearchById(researchId);

    if(!research) {
        return new Response(
            JSON.stringify({
                code: 400,
                message: "Bad Request",
                error: "Invalid research id"
            }), { status: 400 }
        )
    }
    let userPlanet: Planet | null;
    try {
        userPlanet = await getPlanetById(new ObjectId(body.planetId));
    } catch(e) {
        return new Response(
            JSON.stringify({
                code: 400,
                message: "Bad Request",
                error: "Invalid planet id"
            }), { status: 400 }
        )
    }
    if(!userPlanet) {
        return new Response(
            JSON.stringify({
                code: 404,
                message: "Not Found",
                error: "Planet not found"
            }), { status: 404 }
        )
    }
    
    // check requirements
    // buildings
    const buildings = userPlanet.buildings;
    research.requirements.buildings.forEach((buildingReq) => {
        if(buildings.filter((building) => building.id === buildingReq.id)[0]?.level ?? 0 < buildingReq.level) {
            return new Response(
                JSON.stringify({
                    code: 400,
                    message: "Bad Request",
                    error: `${buildingReq.id} level ${buildingReq.level} required, found ${buildings.filter((building) => building.id === buildingReq.id)[0]?.level ?? 0}`
                }), { status: 400 }
            )
        }
    });

    // research
    const playerResearch = await getUserResearch(user);
    research.requirements.research.forEach((researchReq) => {
        if(playerResearch.filter((research) => research.id === researchReq.id)[0].level < researchReq.level) {
            return new Response(
                JSON.stringify({
                    code: 400,
                    message: "Bad Request",
                    error: `${researchReq.id} level ${researchReq.level} required, found ${playerResearch.filter((research) => research.id === researchReq.id)[0].level}`
                }), { status: 400 }
            )
        }
    });

    // resources
    const resources = await calculateCurrentAvailableResources(userPlanet._id);
    if(!resources) {
        return new Response(
            JSON.stringify({
                code: 500,
                message: "Internal Server Error",
                error: "Failed to get resources"
            }), { status: 500 }
        )
    }
    const playerCurrentResearch = playerResearch.filter((element: any) => element.id === researchId)[0];
    const level = playerCurrentResearch ? playerCurrentResearch.level : 0;
    const newResources = structuredClone(resources);
    const missingResources: Array<{}> = [];
    research.requirements.resources.forEach(resource => {
        const res = resources.filter((element: PlayerResource) => element.name === resource.name)[0];
        const cost = playerCurrentResearch ? resource.amount * Math.pow(research.multiplier, level) : resource.amount;
        
        if(res.amount < cost) {
            missingResources.push({
                name: resource.name,
                required: cost,
                available: res.amount
            });
            return;
        }
        else newResources.filter((element: PlayerResource) => element.name === resource.name)[0].amount -= cost;
    });

    if(missingResources.length > 0) {
        return new Response(
            JSON.stringify({
                code: 400,
                message: "Bad Request",
                data: missingResources
            }), { status: 400 }
        )
    }

    await updatePlanetResources(userPlanet._id, newResources);
    await createOrUpgradeResearch(user._id, { id: researchId, level: level + 1 });

    return new Response(
        JSON.stringify({
            code: 200,
            message: "OK"
        }), { status: 200 }
    )
}